Skip site navigation (1)Skip section navigation (2)
Date:      Sun, 13 Dec 2009 18:42:06 +0000 (UTC)
From:      Marius Strobl <marius@FreeBSD.org>
To:        src-committers@freebsd.org, svn-src-all@freebsd.org, svn-src-head@freebsd.org
Subject:   svn commit: r200482 - head/sys/dev/ata/chipsets
Message-ID:  <200912131842.nBDIg67B094157@svn.freebsd.org>

next in thread | raw e-mail | index | archive | help
Author: marius
Date: Sun Dec 13 18:42:06 2009
New Revision: 200482
URL: http://svn.freebsd.org/changeset/base/200482

Log:
  Properly support M5229 revision 0xc7 and 0xc8:
  - These revisions no longer have cable detection capability.
  - The UDMA support bit of register 0x4b has been dropped without an
    replacement.
  - According to Linux it's crucial for working ATAPI DMA support to
    also set the reserved bit 1 of regsiter 0x53 with these revisions.
  
  MFC after:	1 week

Modified:
  head/sys/dev/ata/chipsets/ata-acerlabs.c

Modified: head/sys/dev/ata/chipsets/ata-acerlabs.c
==============================================================================
--- head/sys/dev/ata/chipsets/ata-acerlabs.c	Sun Dec 13 18:26:19 2009	(r200481)
+++ head/sys/dev/ata/chipsets/ata-acerlabs.c	Sun Dec 13 18:42:06 2009	(r200482)
@@ -141,11 +141,14 @@ ata_ali_chipinit(device_t dev)
 	/* use device interrupt as byte count end */
 	pci_write_config(dev, 0x4a, pci_read_config(dev, 0x4a, 1) | 0x20, 1);
 
-	/* enable cable detection and UDMA support on newer chips */
-	pci_write_config(dev, 0x4b, pci_read_config(dev, 0x4b, 1) | 0x09, 1);
-
-	/* enable ATAPI UDMA mode */
-	pci_write_config(dev, 0x53, pci_read_config(dev, 0x53, 1) | 0x01, 1);
+	/* enable cable detection and UDMA support on revisions < 0xc7 */
+	if (ctlr->chip->chiprev < 0xc7)
+	    pci_write_config(dev, 0x4b, pci_read_config(dev, 0x4b, 1) |
+		0x09, 1);
+
+	/* enable ATAPI UDMA mode (even if we are going to do PIO) */
+	pci_write_config(dev, 0x53, pci_read_config(dev, 0x53, 1) |
+	    (ctlr->chip->chiprev >= 0xc7 ? 0x03 : 0x01), 1);
 
 	/* only chips with revision > 0xc4 can do 48bit DMA */
 	if (ctlr->chip->chiprev <= 0xc4)
@@ -276,7 +279,7 @@ ata_ali_setmode(device_t dev, int target
 
         mode = min(mode, ctlr->chip->max_dma);
 
-	if (ctlr->chip->cfg2 & ALI_NEW) {
+	if (ctlr->chip->cfg2 & ALI_NEW && ctlr->chip->chiprev < 0xc7) {
 		if (mode > ATA_UDMA2 &&
 		    pci_read_config(parent, 0x4a, 1) & (1 << ch->unit)) {
 			ata_print_cable(dev, "controller");



Want to link to this message? Use this URL: <https://mail-archive.FreeBSD.org/cgi/mid.cgi?200912131842.nBDIg67B094157>